/**
  ******************************************************************************
  * @file    app.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    31-January-2014
  * @brief   This file provides all the Application firmware functions.
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT 2014 STMicroelectronics</center></h2>
  *
  * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
  * You may not use this file except in compliance with the License.
  * You may obtain a copy of the License at:
  *
  *        http://www.st.com/software_license_agreement_liberty_v2
  *
  * Unless required by applicable law or agreed to in writing, software 
  * distributed under the License is distributed on an "AS IS" BASIS, 
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
  ******************************************************************************
  */ 

#include "usbd_cdc_core.h"
#include "usbd_usr.h"
#include "command.h"
#include "usb_tx_buffer.h"
#include "buzzer.h"
#include <string.h>

USB_CORE_HANDLE  USB_Device_dev ;


int buzz = 0;
static const char szId[] = "miniscope_v2e 20150214";

enum E_ACTION_REQUEST action = REQUEST_NONE;

uint16_t *new_data;

void OnUsbDataRx(uint8_t* dataIn, uint32_t length) {
	if (length>=3)
	{
		if (dataIn[2]==0x00)	//end of message detected - correct frame
		{
			enum E_ACTION_REQUEST tmp = ExecuteCmd(dataIn, length);
			if (tmp != REQUEST_NONE)
			{
				action = tmp;
			}
		}
		length=0;
	}	
}


void UsbSendData(const unsigned char* data, int len)
{
	int i;
	for (i = 0; i < len; i++)
	{
		USART_Rx_Buffer[USART_Rx_ptr_in] = data[i];
		USART_Rx_ptr_in++;

		/* To avoid buffer overflow */
		if (USART_Rx_ptr_in == USART_RX_DATA_SIZE)
		{
			USART_Rx_ptr_in = 0;
		}
	}
}


/** @brief  This function enables the clock in the RCC for the ADC
  */
__INLINE void SetClockForADC(void)
{
  /* (1) Enable the peripheral clock of the ADC */
  RCC->APB2ENR |= RCC_APB2ENR_ADC1EN; /* (1) */
}

/**
  * @brief  This function performs a self-calibration of the ADC
  */
__INLINE void  CalibrateADC(void)
{
  /* (1) Ensure that ADEN = 0 */
  /* (2) Clear ADEN */ 
  /* (3) Launch the calibration by setting ADCAL */
  /* (4) Wait until ADCAL=0 */
  if ((ADC1->CR & ADC_CR_ADEN) != 0) /* (1) */
	{
    ADC1->CR &= (uint32_t)(~ADC_CR_ADEN);  /* (2) */  
  }
  ADC1->CR |= ADC_CR_ADCAL; /* (3) */
  while ((ADC1->CR & ADC_CR_ADCAL) != 0) /* (4) */
  {
    /* For robust implementation, add here time-out management */
  }
}

__INLINE void  ConfigureGPIOforADC(void)
{
  /* Select analog mode for PA0 */
  GPIOA->MODER |= GPIO_MODER_MODER0;
}

/**
  * @brief  This function enables the ADC
  */
__INLINE void EnableADC(void)
{
  /* (1) Enable the ADC */
  /* (2) Wait until ADC ready */
  do 
  {
    /* For robust implementation, add here time-out management */
		ADC1->CR |= ADC_CR_ADEN; /* (1) */
  }while ((ADC1->ISR & ADC_ISR_ADRDY) == 0) /* (2) */;
}

/**
  * @brief  This function configures the ADC to convert sequentially 4 channels
            in continuous mode.
  *         PCLK = HCLK = SYSCLK = 48MHz; use PCLK/4
  *         The interrupt on overrun is enabled and the NVIC is configured
  */
__INLINE void ConfigureADC(void)
{
  /* (1) Select PCLK/4 CKMODE */ 
  /* (2) Select the continuous mode and data align: left */
  /* (3) Select CHSEL0 = PA0 */
  /* (4) Select a sampling time (1.5, 7.5, 13.5, etc. clocks) */
  ADC1->CFGR2 |= ADC_CFGR2_CKMODE_1; /* (1) */
	// tSAR = 9.5 cycles at 8bits/sample
	// tSAR = 11.5 cycles at 10bits/sample
	// tSAR = 12.5 cycles at 12bits/sample
  ADC1->CFGR1 |= ADC_CFGR1_CONT | ADC_CFGR1_ALIGN  | ADC_CFGR1_RES_1  /* 8bit res */ ; /* (2)*/
  ADC1->CHSELR = ADC_CHSELR_CHSEL0; /* (3)*/
//  ADC1->SMPR |= ADC_SMPR_SMP_0 /* | ADC_SMPR_SMP_1 | ADC_SMPR_SMP_2 */; /* (4) */
  ADC1->SMPR |= ADC_SMPR_SMP_1; /* (4) */

}

uint16_t ADC_array[19*64/2];

/**
  * @brief  This function configures the DMA to store the result of an ADC sequence.
  *         The conversion results are stored in N-items array.
  */
__INLINE void ConfigureDMA(void)
{
  /* (1) Enable the peripheral clock on DMA */
  /* (2) Enable DMA transfer on ADC and circular mode */ 
  /* (3) Configure the peripheral data register address */ 
  /* (4) Configure the memory address */
  /* (5) Configure the number of DMA tranfer to be performs on DMA channel 1 */
  /* (6) Configure increment, size, interrupts and circular mode */
  /* (7) Enable DMA Channel 1 */
  RCC->AHBENR |= RCC_AHBENR_DMA1EN; /* (1) */
  ADC1->CFGR1 |= ADC_CFGR1_DMAEN | ADC_CFGR1_DMACFG; /* (2) */
  DMA1_Channel1->CPAR = (uint32_t) (&(ADC1->DR)); /* (3) */
  DMA1_Channel1->CMAR = (uint32_t)(ADC_array); /* (4) */
  DMA1_Channel1->CNDTR = sizeof(ADC_array)/sizeof(ADC_array[0]); /* (5) */
  DMA1_Channel1->CCR |= DMA_CCR_MINC /* mem inc mode */ | DMA_CCR_MSIZE_0 | DMA_CCR_PSIZE_0 \
                      | DMA_CCR_TCIE | DMA_CCR_HTIE | DMA_CCR_CIRC; /* (6) */  
  DMA1_Channel1->CCR |= DMA_CCR_EN; /* (7) */
 
  /* Configure NVIC for DMA */
  /* (8) Enable Interrupt on DMA */
  /* (9) Set priority for DMA */
  NVIC_EnableIRQ(DMA1_Channel1_IRQn); /* (8) */
  NVIC_SetPriority(DMA1_Channel1_IRQn,0); /* (9) */
}

void DMA1_Channel1_IRQHandler(void)
{
	unsigned short *ptr;
#if 0	
	{
		static int cnt = 0;
		if (++cnt >= 10000) {
			buzz += 100000;
			cnt = 0;
		}
	}
#endif	
	if(DMA1->ISR & DMA1_IT_HT1)
	{
		/* Half transfer - 1st half buffer has data */
		ptr = &ADC_array[0];
	}
	else
	{
		/* Transfer complete - 2nd half buffer has data */
		ptr = &ADC_array[sizeof(ADC_array)/sizeof(ADC_array[0])/2];
	}

	/* process the buffer */
	new_data = ptr;
	
	/* Clear DMA1_Channel1 interrupt */
	DMA1->IFCR = DMA_IFCR_CGIF1;
}

int main(void)
{
	unsigned short *data = NULL;
	int identified = 0;
  /*!< At this stage the microcontroller clock setting is already configured, 
  this is done through SystemInit() function which is called from startup
  file (startup_stm32f0x2.s) before to branch to application main.
  To reconfigure the default setting of SystemInit() function, refer to
  system_stm32f0xx.c file
  */  
	
	// enable peripherial clock for GPIOA
  RCC->AHBENR |= RCC_AHBENR_GPIOAEN;
	
	SetCdcDataReceiveCb(OnUsbDataRx);
	
#if defined (STM32F042)
	RCC->APB2ENR |= RCC_APB2ENR_SYSCFGCOMPEN;	
	// remap pins to enable USB
	SYSCFG->CFGR1 |= SYSCFG_CFGR1_PA11_PA12_RMP;
#endif
  
	Buzzer_Init();
	Beep();	
	
  SetClockForADC();			
  CalibrateADC(); 
  ConfigureGPIOforADC();
  EnableADC();	
  ConfigureADC();
  ConfigureDMA();

  /* The Application layer has only to call USBD_Init to 
  initialize the USB low level driver, the USB device library, the USB clock 
  ,pins and interrupt service routine (BSP) to start the Library*/
  USBD_Init(&USB_Device_dev,
            &USR_desc, 
            &USBD_CDC_cb, 
            &USR_cb);
  
  ADC1->CR |= ADC_CR_ADSTART; /* start the ADC conversions */
	
  while (1)
  {
		if (buzz > 0)
		{
			buzz--;
			if (buzz) {
				Buzzer_On();
			} else {
				Buzzer_Off();				
			}
		}

		if (action == REQUEST_IDENTIFY)
		{
			int len = 0, i;
			action = REQUEST_NONE;

			buzz += 30000;
			
			while (len < 19*64)
			{
				UsbSendData((const unsigned char*)szId, strlen(szId));
				len += strlen(szId);
			}
			// pad reply to same size as regular data packet
			for (i = 0; i < (19*64) - len; i++)
			{
				USART_Rx_Buffer[USART_Rx_ptr_in++] = 0;
				if (USART_Rx_ptr_in == USART_RX_DATA_SIZE)
				{
					USART_Rx_ptr_in = 0;
				}
			}
			identified = 1;
		}

		if (identified && new_data != data)
		{
			int i;
			unsigned char *ptr;
			data = new_data;
			
			ptr = (unsigned char*)data + 1;
			for (i = 0; i < sizeof(ADC_array)/sizeof(ADC_array[0])/2; i++)
			{
				USART_Rx_Buffer[USART_Rx_ptr_in++] = *ptr; //*(((unsigned char*)&data[i]) + 1);
				ptr += 2;
				if (USART_Rx_ptr_in == USART_RX_DATA_SIZE)
				{
					USART_Rx_ptr_in = 0;
				}
#if 1
				if (USART_Rx_ptr_out == USART_Rx_ptr_in)
				{
					// USB FIFO overflow debug
					if (buzz < 2000) {
						buzz += 2000;
					}
				}
#endif					
			}		
		}
  }
} 

#ifdef USE_FULL_ASSERT
/**
  * @brief  assert_failed
  *         Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  File: pointer to the source file name
  * @param  Line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
  ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  Buzzer_On();
  /* Infinite loop */
  while (1)
  {}
}
#endif

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
